@@ -1,5 +1,5 @@
 /*
- * Copyright 2010 Canonical Ltd.
+ * Copyright 2010-2012 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of either or both of the following licenses:
@@ -21,6 +21,7 @@
  *
  * Authored by: Jason Smith <jason.smith@canonical.com>
  *              Neil Jagdish Patel <neil.patel@canonical.com>
+ *              Marco Trevisan (Treviño) <3v1n0@ubuntu.com>
  *
  */
 /**
@@ -34,10 +35,9 @@
 #include <config.h>
 #endif
 
+#include <libbamf-private/bamf-private.h>
 #include "bamf-control.h"
-
-#include <dbus/dbus.h>
-#include <dbus/dbus-glib.h>
+#include "bamf-view-private.h"
 
 G_DEFINE_TYPE (BamfControl, bamf_control, G_TYPE_OBJECT);
 
@@ -46,8 +46,7 @@ G_DEFINE_TYPE (BamfControl, bamf_control, G_TYPE_OBJECT);
 
 struct _BamfControlPrivate
 {
-  DBusGConnection *connection;
-  DBusGProxy      *proxy;
+  BamfDBusControl *proxy;
 };
 
 /* Globals */
@@ -70,20 +69,28 @@ bamf_control_dispose (GObject *object)
       self->priv->proxy = NULL;
     }
 
-  if (G_OBJECT_CLASS (bamf_control_parent_class)->dispose)
-      G_OBJECT_CLASS (bamf_control_parent_class)->dispose (object);
+  G_OBJECT_CLASS (bamf_control_parent_class)->dispose (object);
+}
+
+static void
+bamf_control_finalize (GObject *object)
+{
+  default_control = NULL;
+
+  G_OBJECT_CLASS (bamf_control_parent_class)->finalize (object);
 }
 
 static void
 bamf_control_class_init (BamfControlClass *klass)
 {
   GObjectClass *obj_class = G_OBJECT_CLASS (klass);
+  obj_class->dispose = bamf_control_dispose;
+  obj_class->finalize = bamf_control_finalize;
 
   g_type_class_add_private (obj_class, sizeof (BamfControlPrivate));
   obj_class->dispose = bamf_control_dispose;
 }
 
-
 static void
 bamf_control_init (BamfControl *self)
 {
@@ -92,30 +99,26 @@ bamf_control_init (BamfControl *self)
 
   priv = self->priv = BAMF_CONTROL_GET_PRIVATE (self);
 
-  priv->connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
-  if (priv->connection == NULL)
+  priv->proxy = _bamf_dbus_control_proxy_new_for_bus_sync (G_BUS_TYPE_SESSION,
+                                                           G_DBUS_PROXY_FLAGS_NONE,
+                                                           BAMF_DBUS_SERVICE_NAME,
+                                                           BAMF_DBUS_CONTROL_PATH,
+                                                           NULL, &error);
+
+  if (error)
     {
-      g_warning ("Failed to open connection to bus: %s",
-               error != NULL ? error->message : "Unknown");
-      if (error)
-        g_error_free (error);
+      g_error ("Unable to get "BAMF_DBUS_CONTROL_PATH" controller: %s", error->message);
+      g_error_free (error);
       return;
     }
 
-  priv->proxy = dbus_g_proxy_new_for_name (priv->connection,
-                                           "org.ayatana.bamf",
-                                           "/org/ayatana/bamf/control",
-                                           "org.ayatana.bamf.control");
-  if (priv->proxy == NULL)
-    {
-      g_error ("Unable to get org.bamf.Control control");
-    }
+  g_dbus_proxy_set_default_timeout (G_DBUS_PROXY (priv->proxy), BAMF_DBUS_DEFAULT_TIMEOUT);
 }
 
 /**
  * bamf_control_get_default:
  *
- * Returns: (transfer none): The default #BamfControl reference.
+ * Returns: (transfer full): The default #BamfControl reference.
  */
 BamfControl *
 bamf_control_get_default (void)
@@ -123,12 +126,13 @@ bamf_control_get_default (void)
   if (BAMF_IS_CONTROL (default_control))
     return g_object_ref (default_control);
 
-  return (default_control = g_object_new (BAMF_TYPE_CONTROL, NULL));
+  default_control = g_object_new (BAMF_TYPE_CONTROL, NULL);
+
+  return default_control;
 }
 
 void
-bamf_control_set_approver_behavior (BamfControl *control,
-                                    gint32       behavior)
+bamf_control_insert_desktop_file (BamfControl *control, const gchar *desktop_file)
 {
   BamfControlPrivate *priv;
   GError *error = NULL;
@@ -136,43 +140,41 @@ bamf_control_set_approver_behavior (BamfControl *control,
   g_return_if_fail (BAMF_IS_CONTROL (control));
   priv = control->priv;
 
-  if (!dbus_g_proxy_call (priv->proxy,
-                          "SetApproverBehavior",
-                          &error,
-                          G_TYPE_INT, behavior,
-                          G_TYPE_INVALID,
-                          G_TYPE_INVALID))
-    {
-      g_warning ("Failed to register application: %s", error->message);
-      g_error_free (error);
-    }
-}
-
-void
-bamf_control_insert_desktop_file (BamfControl *control,
-                                   const gchar *desktop_file)
-{
-  BamfControlPrivate *priv;
-  GError *error = NULL;
-
-  g_return_if_fail (BAMF_IS_CONTROL (control));
-  priv = control->priv;
-
-  if (!dbus_g_proxy_call (priv->proxy,
-                          "OmNomNomDesktopFile",
-                          &error,
-                          G_TYPE_STRING, desktop_file,
-                          G_TYPE_INVALID,
-                          G_TYPE_INVALID))
+  if (!_bamf_dbus_control_call_insert_desktop_file_sync (priv->proxy, desktop_file,
+                                                         NULL, &error))
     {
       g_warning ("Failed to insert desktop file: %s", error->message);
       g_error_free (error);
     }
 }
 
+void
+bamf_control_create_local_desktop_file (BamfControl *control, BamfApplication *app)
+{
+  BamfControlPrivate *priv;
+  const gchar *app_path;
+  GError *error = NULL;
+
+  g_return_if_fail (BAMF_IS_CONTROL (control));
+  g_return_if_fail (BAMF_IS_APPLICATION (app));
+
+  priv = control->priv;
+  app_path = _bamf_view_get_path (BAMF_VIEW (app));
+
+  if (!app_path)
+    return;
+
+  if (!_bamf_dbus_control_call_create_local_desktop_file_sync (priv->proxy, app_path,
+                                                               NULL, &error))
+    {
+      g_warning ("Failed to create local desktop file: %s", error->message);
+      g_error_free (error);
+    }
+}
+
 void
 bamf_control_register_application_for_pid (BamfControl  *control,
-                                           const gchar  *application,
+                                           const gchar  *desktop_file,
                                            gint32        pid)
 {
   BamfControlPrivate *priv;
@@ -181,21 +183,12 @@ bamf_control_register_application_for_pid (BamfControl  *control,
   g_return_if_fail (BAMF_IS_CONTROL (control));
   priv = control->priv;
 
-  if (!dbus_g_proxy_call (priv->proxy,
-                          "RegisterApplicationForPid",
-                          &error,
-                          G_TYPE_STRING, application,
-                          G_TYPE_INT, pid,
-                          G_TYPE_INVALID,
-                          G_TYPE_INVALID))
+  if (!_bamf_dbus_control_call_register_application_for_pid_sync (priv->proxy,
+                                                                  desktop_file,
+                                                                  pid, NULL,
+                                                                  &error))
     {
       g_warning ("Failed to register application: %s", error->message);
       g_error_free (error);
     }
 }
-
-void
-bamf_control_register_tab_provider (BamfControl *control,
-                                    const char  *path)
-{
-}
