@@ -1,5 +1,5 @@
 /*
- * Copyright 2010 Canonical Ltd.
+ * Copyright 2010-2012 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of either or both of the following licenses:
@@ -21,6 +21,7 @@
  *
  * Authored by: Jason Smith <jason.smith@canonical.com>
  *              Neil Jagdish Patel <neil.patel@canonical.com>
+ *              Marco Trevisan (Treviño) <3v1n0@ubuntu.com>
  *
  */
 /**
@@ -34,13 +35,10 @@
 #include <config.h>
 #endif
 
+#include <libbamf-private/bamf-private.h>
 #include "bamf-view-private.h"
 #include "bamf-window.h"
 #include "bamf-factory.h"
-#include "bamf-marshal.h"
-#include <dbus/dbus.h>
-#include <dbus/dbus-glib.h>
-#include <dbus/dbus-glib-lowlevel.h>
 
 G_DEFINE_TYPE (BamfWindow, bamf_window, BAMF_TYPE_VIEW);
 
@@ -49,12 +47,12 @@ G_DEFINE_TYPE (BamfWindow, bamf_window, BAMF_TYPE_VIEW);
 
 struct _BamfWindowPrivate
 {
-  DBusGConnection *connection;
-  DBusGProxy      *proxy;
-  guint32          xid;
-  guint32          pid;
-  time_t           last_active;
-  gint             monitor;
+  BamfDBusItemWindow        *proxy;
+  guint32                    xid;
+  guint32                    pid;
+  time_t                     last_active;
+  gint                       monitor;
+  BamfWindowType             type;
   BamfWindowMaximizationType maximized;
 };
 
@@ -68,10 +66,14 @@ enum
 
 static guint window_signals[LAST_SIGNAL] = { 0 };
 
-time_t bamf_window_last_active (BamfWindow *self)
+time_t
+bamf_window_last_active (BamfWindow *self)
 {
-  g_return_val_if_fail (BAMF_IS_WINDOW (self), (time_t) 0);
-  
+  g_return_val_if_fail (BAMF_IS_WINDOW (self), 0);
+
+  if (BAMF_WINDOW_GET_CLASS (self)->last_active)
+    return BAMF_WINDOW_GET_CLASS (self)->last_active (self);
+
   return self->priv->last_active;
 }
 
@@ -82,7 +84,8 @@ time_t bamf_window_last_active (BamfWindow *self)
  * Returns: (transfer none) (allow-none): A transient for this #BamfWindow.
  */
 
-BamfWindow * bamf_window_get_transient (BamfWindow *self)
+BamfWindow *
+bamf_window_get_transient (BamfWindow *self)
 {
   BamfWindowPrivate *priv;
   BamfView *transient;
@@ -90,225 +93,130 @@ BamfWindow * bamf_window_get_transient (BamfWindow *self)
   GError *error = NULL;
 
   g_return_val_if_fail (BAMF_IS_WINDOW (self), FALSE);
+
+  if (BAMF_WINDOW_GET_CLASS (self)->get_transient)
+    return BAMF_WINDOW_GET_CLASS (self)->get_transient (self);
+
   priv = self->priv;
 
   if (!_bamf_view_remote_ready (BAMF_VIEW (self)))
     return NULL;
 
-  if (!dbus_g_proxy_call (priv->proxy,
-                          "Transient",
-                          &error,
-                          G_TYPE_INVALID,
-                          G_TYPE_STRING, &path,
-                          G_TYPE_INVALID))
+  if (!_bamf_dbus_item_window_call_transient_sync (priv->proxy, &path,
+                                                   CANCELLABLE (self), &error))
     {
-      g_warning ("Failed to fetch path: %s", error->message);
+      g_warning ("Failed to fetch path: %s", error ? error->message : "");
       g_error_free (error);
-      return 0;
+      return NULL;
     }
-  
+
   if (!path)
     return NULL;
 
+  if (path[0] == '\0')
+    {
+      g_free (path);
+      return NULL;
+    }
+
   BamfFactory *factory = _bamf_factory_get_default ();
   transient = _bamf_factory_view_for_path_type (factory, path, BAMF_FACTORY_WINDOW);
-  g_free (path);  
+  g_free (path);
 
   if (!BAMF_IS_WINDOW (transient))
     return NULL;
-    
+
   return BAMF_WINDOW (transient);
 }
 
-BamfWindowType bamf_window_get_window_type (BamfWindow *self)
+BamfWindowType
+bamf_window_get_window_type (BamfWindow *self)
 {
   BamfWindowPrivate *priv;
-  guint32 type = 0;
   GError *error = NULL;
 
   g_return_val_if_fail (BAMF_IS_WINDOW (self), FALSE);
+
+  if (BAMF_WINDOW_GET_CLASS (self)->get_window_type)
+    return BAMF_WINDOW_GET_CLASS (self)->get_window_type (self);
+
   priv = self->priv;
 
-  if (!_bamf_view_remote_ready (BAMF_VIEW (self)))
-    return 0;
+  if (priv->type != BAMF_WINDOW_UNKNOWN)
+    return priv->type;
 
-  if (!dbus_g_proxy_call (priv->proxy,
-                          "WindowType",
-                          &error,
-                          G_TYPE_INVALID,
-                          G_TYPE_UINT, &type,
-                          G_TYPE_INVALID))
+  if (!_bamf_view_remote_ready (BAMF_VIEW (self)))
+    return priv->type;
+
+  if (!_bamf_dbus_item_window_call_window_type_sync (priv->proxy, &priv->type,
+                                                     CANCELLABLE (self), &error))
     {
-      g_warning ("Failed to fetch type: %s", error->message);
+      priv->type = BAMF_WINDOW_UNKNOWN;
+      g_warning ("Failed to fetch type: %s", error ? error->message : "");
       g_error_free (error);
-      return 0;
     }
 
-  return (BamfWindowType) type;
+  return priv->type;
 }
 
-guint32 bamf_window_get_pid (BamfWindow *self)
+guint32
+bamf_window_get_pid (BamfWindow *self)
 {
   BamfWindowPrivate *priv;
-  guint32 pid = 0;
   GError *error = NULL;
 
   g_return_val_if_fail (BAMF_IS_WINDOW (self), FALSE);
+
+  if (BAMF_WINDOW_GET_CLASS (self)->get_pid)
+    return BAMF_WINDOW_GET_CLASS (self)->get_pid (self);
+
   priv = self->priv;
 
   if (priv->pid != 0)
     return priv->pid;
 
   if (!_bamf_view_remote_ready (BAMF_VIEW (self)))
-    return 0;
+    return priv->pid;
 
-  if (!dbus_g_proxy_call (priv->proxy,
-                          "GetPid",
-                          &error,
-                          G_TYPE_INVALID,
-                          G_TYPE_UINT, &pid,
-                          G_TYPE_INVALID))
+  if (!_bamf_dbus_item_window_call_get_pid_sync (priv->proxy, &priv->pid,
+                                                 CANCELLABLE (self), &error))
     {
-      g_warning ("Failed to fetch pid: %s", error->message);
+      priv->pid = 0;
+      g_warning ("Failed to fetch pid: %s", error ? error->message : "");
       g_error_free (error);
-      return 0;
     }
 
-  return pid;
+  return priv->pid;
 }
 
-guint32 bamf_window_get_xid (BamfWindow *self)
+guint32
+bamf_window_get_xid (BamfWindow *self)
 {
   BamfWindowPrivate *priv;
-  guint32 xid = 0;
   GError *error = NULL;
-  
+
   g_return_val_if_fail (BAMF_IS_WINDOW (self), FALSE);
+
+  if (BAMF_WINDOW_GET_CLASS (self)->get_xid)
+    return BAMF_WINDOW_GET_CLASS (self)->get_xid (self);
+
   priv = self->priv;
-  
+
   if (priv->xid != 0)
     return priv->xid;
 
   if (!_bamf_view_remote_ready (BAMF_VIEW (self)))
-    return 0;
+    return priv->xid;
 
-  if (!dbus_g_proxy_call (priv->proxy,
-                          "GetXid",
-                          &error,
-                          G_TYPE_INVALID,
-                          G_TYPE_UINT, &xid,
-                          G_TYPE_INVALID))
+  if (!_bamf_dbus_item_window_call_get_xid_sync (priv->proxy, &priv->xid,
+                                                 CANCELLABLE (self), &error))
     {
-      g_warning ("Failed to fetch xid: %s", error->message);
+      priv->xid = 0;
+      g_warning ("Failed to fetch xid: %s", error ? error->message : "");
       g_error_free (error);
-      return 0;
     }
 
-  return xid;
-}
-
-static void
-bamf_window_active_changed (BamfView *view, gboolean active)
-{
-  BamfWindow *self;
-
-  g_return_if_fail (BAMF_IS_WINDOW (view));
-
-  self = BAMF_WINDOW (view);
-  
-  if (active)
-    self->priv->last_active = time (NULL);
-}
-
-static void
-bamf_window_on_monitor_changed (DBusGProxy *proxy, gint old, gint new, BamfWindow *self)
-{
-  self->priv->monitor = new;
-  g_signal_emit (G_OBJECT (self), window_signals[MONITOR_CHANGED], 0, old, new);
-}
-
-static void
-bamf_window_on_maximized_changed (DBusGProxy *proxy, gint old, gint new, BamfWindow *self)
-{
-  self->priv->maximized = new;
-  g_signal_emit (G_OBJECT (self), window_signals[MAXIMIZED_CHANGED], 0, old, new);
-}
-
-static void
-bamf_window_unset_proxy (BamfWindow *self)
-{
-  BamfWindowPrivate *priv;
-
-  g_return_if_fail (BAMF_IS_WINDOW (self));
-  priv = self->priv;
-
-  if (!priv->proxy)
-    return;
-
-  dbus_g_proxy_disconnect_signal (self->priv->proxy,
-                                  "MaximizedChanged",
-                                  (GCallback) bamf_window_on_maximized_changed,
-                                  self);
-
-  dbus_g_proxy_disconnect_signal (self->priv->proxy,
-                                  "MonitorChanged",
-                                  (GCallback) bamf_window_on_monitor_changed,
-                                  self);
-
-  g_object_unref (priv->proxy);
-  priv->proxy = NULL;
-}
-
-static void
-bamf_window_set_path (BamfView *view, const char *path)
-{
-  BamfWindow *self;
-  BamfWindowPrivate *priv;
-  
-  self = BAMF_WINDOW (view);
-  priv = self->priv;
-
-  bamf_window_unset_proxy (self);
-  priv->proxy = dbus_g_proxy_new_for_name (priv->connection,
-                                           "org.ayatana.bamf",
-                                           path,
-                                           "org.ayatana.bamf.window");
-  if (priv->proxy == NULL)
-    {
-      g_error ("Unable to get org.ayatana.bamf.window window");
-      return;
-    }
-
-  priv->xid = bamf_window_get_xid (self);
-  priv->monitor = bamf_window_get_monitor (self);
-  priv->maximized = bamf_window_maximized (self);
-
-  dbus_g_object_register_marshaller ((GClosureMarshal) _bamf_marshal_VOID__INT_INT,
-                                     G_TYPE_NONE, 
-                                     G_TYPE_INT, G_TYPE_INT,
-                                     G_TYPE_INVALID);
-
-  dbus_g_proxy_add_signal (priv->proxy,
-                           "MonitorChanged",
-                           G_TYPE_INT,
-                           G_TYPE_INT,
-                           G_TYPE_INVALID);
-
-  dbus_g_proxy_connect_signal (priv->proxy,
-                               "MonitorChanged",
-                               (GCallback) bamf_window_on_monitor_changed,
-                               self, NULL);
-
-  dbus_g_proxy_add_signal (priv->proxy,
-                           "MaximizedChanged",
-                           G_TYPE_INT,
-                           G_TYPE_INT,
-                           G_TYPE_INVALID);
-
-  dbus_g_proxy_connect_signal (priv->proxy,
-                               "MaximizedChanged",
-                               (GCallback) bamf_window_on_maximized_changed,
-                               self, NULL);
+  return priv->xid;
 }
 
 gchar *
@@ -319,20 +227,20 @@ bamf_window_get_utf8_prop (BamfWindow *self, const char* xprop)
   GError *error = NULL;
 
   g_return_val_if_fail (BAMF_IS_WINDOW (self), NULL);
+  g_return_val_if_fail (xprop, NULL);
+
+  if (BAMF_WINDOW_GET_CLASS (self)->get_utf8_prop)
+    return BAMF_WINDOW_GET_CLASS (self)->get_utf8_prop (self, xprop);
+
   priv = self->priv;
 
   if (!_bamf_view_remote_ready (BAMF_VIEW (self)))
     return NULL;
 
-  if (!dbus_g_proxy_call (priv->proxy,
-                          "Xprop",
-                          &error,
-                          G_TYPE_STRING, xprop,
-                          G_TYPE_INVALID,
-                          G_TYPE_STRING, &result,
-                          G_TYPE_INVALID))
+  if (!_bamf_dbus_item_window_call_xprop_sync (priv->proxy, xprop, &result,
+                                               CANCELLABLE (self), &error))
     {
-      g_warning ("Failed to fetch property: %s", error->message);
+      g_warning ("Failed to fetch property `%s': %s", xprop, error ? error->message : "");
       g_error_free (error);
 
       return NULL;
@@ -355,6 +263,10 @@ bamf_window_get_monitor (BamfWindow *self)
   GError *error = NULL;
 
   g_return_val_if_fail (BAMF_IS_WINDOW (self), -1);
+
+  if (BAMF_WINDOW_GET_CLASS (self)->get_monitor)
+    return BAMF_WINDOW_GET_CLASS (self)->get_monitor (self);
+
   priv = self->priv;
 
   if (priv->monitor != -2 || !_bamf_view_remote_ready (BAMF_VIEW (self)))
@@ -362,17 +274,13 @@ bamf_window_get_monitor (BamfWindow *self)
       return priv->monitor;
     }
 
-  if (!dbus_g_proxy_call (priv->proxy,
-                          "Monitor",
-                          &error,
-                          G_TYPE_INVALID,
-                          G_TYPE_INT, &monitor,
-                          G_TYPE_INVALID))
+  if (!_bamf_dbus_item_window_call_monitor_sync (priv->proxy, &monitor,
+                                                 CANCELLABLE (self), &error))
     {
-      g_warning ("Failed to fetch monitor: %s", error->message);
+      g_warning ("Failed to fetch monitor: %s", error ? error->message : "");
       g_error_free (error);
 
-      return -1;
+      monitor = -1;
     }
 
   return monitor;
@@ -382,10 +290,13 @@ BamfWindowMaximizationType
 bamf_window_maximized (BamfWindow *self)
 {
   BamfWindowPrivate *priv;
-  BamfWindowMaximizationType maximized = -1;
   GError *error = NULL;
 
   g_return_val_if_fail (BAMF_IS_WINDOW (self), -1);
+
+  if (BAMF_WINDOW_GET_CLASS (self)->maximized)
+    return BAMF_WINDOW_GET_CLASS (self)->maximized (self);
+
   priv = self->priv;
 
   if (priv->maximized != -1 || !_bamf_view_remote_ready (BAMF_VIEW (self)))
@@ -393,28 +304,102 @@ bamf_window_maximized (BamfWindow *self)
       return priv->maximized;
     }
 
-  if (!dbus_g_proxy_call (priv->proxy,
-                          "Maximized",
-                          &error,
-                          G_TYPE_INVALID,
-                          G_TYPE_INT, &maximized,
-                          G_TYPE_INVALID))
+  if (!_bamf_dbus_item_window_call_maximized_sync (priv->proxy, (gint*) &priv->maximized,
+                                                   CANCELLABLE (self), &error))
     {
+      priv->maximized = -1;
       g_warning ("Failed to fetch maximized state: %s", error->message);
       g_error_free (error);
-
-      return -1;
     }
 
-  return maximized;
+  return priv->maximized;
+}
+
+static void
+bamf_window_active_changed (BamfView *view, gboolean active)
+{
+  BamfWindow *self;
+
+  g_return_if_fail (BAMF_IS_WINDOW (view));
+
+  self = BAMF_WINDOW (view);
+
+  if (active)
+    self->priv->last_active = time (NULL);
+}
+
+static void
+bamf_window_on_monitor_changed (BamfDBusItemWindow *proxy, gint old, gint new, BamfWindow *self)
+{
+  self->priv->monitor = new;
+  g_signal_emit (G_OBJECT (self), window_signals[MONITOR_CHANGED], 0, old, new);
+}
+
+static void
+bamf_window_on_maximized_changed (BamfDBusItemWindow *proxy, gint old, gint new, BamfWindow *self)
+{
+  self->priv->maximized = new;
+  g_signal_emit (G_OBJECT (self), window_signals[MAXIMIZED_CHANGED], 0, old, new);
+}
+
+static void
+bamf_window_unset_proxy (BamfWindow *self)
+{
+  BamfWindowPrivate *priv;
+
+  g_return_if_fail (BAMF_IS_WINDOW (self));
+  priv = self->priv;
+
+  if (!G_IS_DBUS_PROXY (priv->proxy))
+    return;
+
+  g_signal_handlers_disconnect_by_data (priv->proxy, self);
+
+  g_object_unref (priv->proxy);
+  priv->proxy = NULL;
+}
+
+static void
+bamf_window_set_path (BamfView *view, const char *path)
+{
+  BamfWindow *self;
+  BamfWindowPrivate *priv;
+  GError *error = NULL;
+
+  self = BAMF_WINDOW (view);
+  priv = self->priv;
+
+  bamf_window_unset_proxy (self);
+  priv->proxy = _bamf_dbus_item_window_proxy_new_for_bus_sync (G_BUS_TYPE_SESSION,
+                                                               G_DBUS_PROXY_FLAGS_NONE,
+                                                               BAMF_DBUS_SERVICE_NAME,
+                                                               path, CANCELLABLE (self),
+                                                               &error);
+  if (!G_IS_DBUS_PROXY (priv->proxy))
+    {
+      g_error ("Unable to get %s window: %s", BAMF_DBUS_SERVICE_NAME, error ? error->message : "");
+      g_error_free (error);
+      return;
+    }
+
+  g_dbus_proxy_set_default_timeout (G_DBUS_PROXY (priv->proxy), BAMF_DBUS_DEFAULT_TIMEOUT);
+
+  priv->xid = bamf_window_get_xid (self);
+  priv->type = bamf_window_get_window_type (self);
+  priv->monitor = bamf_window_get_monitor (self);
+  priv->maximized = bamf_window_maximized (self);
+
+  g_signal_connect (priv->proxy, "monitor-changed",
+                    G_CALLBACK (bamf_window_on_monitor_changed), self);
+
+  g_signal_connect (priv->proxy, "maximized-changed",
+                    G_CALLBACK (bamf_window_on_maximized_changed), self);
 }
 
 static void
 bamf_window_dispose (GObject *object)
 {
-  BamfWindow *self;
-  
-  self = BAMF_WINDOW (object);
+  BamfWindow *self = BAMF_WINDOW (object);
   bamf_window_unset_proxy (self);
 
   if (G_OBJECT_CLASS (bamf_window_parent_class)->dispose)
@@ -428,28 +413,26 @@ bamf_window_class_init (BamfWindowClass *klass)
   BamfViewClass *view_class = BAMF_VIEW_CLASS (klass);
 
   g_type_class_add_private (obj_class, sizeof (BamfWindowPrivate));
-  
-  obj_class->dispose     = bamf_window_dispose;
+
+  obj_class->dispose = bamf_window_dispose;
   view_class->active_changed = bamf_window_active_changed;
-  view_class->set_path   = bamf_window_set_path;
-  
-  window_signals[MONITOR_CHANGED] = 
-    g_signal_new ("monitor-changed",
+  view_class->set_path = bamf_window_set_path;
+
+  window_signals[MONITOR_CHANGED] =
+    g_signal_new (BAMF_WINDOW_SIGNAL_MONITOR_CHANGED,
                   G_OBJECT_CLASS_TYPE (klass),
                   G_SIGNAL_RUN_FIRST,
                   G_STRUCT_OFFSET (BamfWindowClass, monitor_changed),
-                  NULL, NULL,
-                  _bamf_marshal_VOID__INT_INT,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 2,
                   G_TYPE_INT, G_TYPE_INT);
 
-  window_signals[MAXIMIZED_CHANGED] = 
-    g_signal_new ("maximized-changed",
+  window_signals[MAXIMIZED_CHANGED] =
+    g_signal_new (BAMF_WINDOW_SIGNAL_MAXIMIZED_CHANGED,
                   G_OBJECT_CLASS_TYPE (klass),
                   G_SIGNAL_RUN_FIRST,
                   G_STRUCT_OFFSET (BamfWindowClass, maximized_changed),
-                  NULL, NULL,
-                  _bamf_marshal_VOID__INT_INT,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 2,
                   G_TYPE_INT, G_TYPE_INT);
 }
@@ -458,32 +441,19 @@ static void
 bamf_window_init (BamfWindow *self)
 {
   BamfWindowPrivate *priv;
-  GError           *error = NULL;
 
   priv = self->priv = BAMF_WINDOW_GET_PRIVATE (self);
-
   priv->xid = 0;
   priv->pid = 0;
+  priv->type = BAMF_WINDOW_UNKNOWN;
   priv->monitor = -2;
   priv->maximized = -1;
-
-  priv->connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
-  if (priv->connection == NULL)
-    {
-      g_critical ("Failed to open connection to bus: %s",
-               error != NULL ? error->message : "Unknown");
-      if (error)
-        g_error_free (error);
-      return;
-    }
 }
 
 BamfWindow *
 bamf_window_new (const char * path)
 {
-  BamfWindow *self;
-  self = g_object_new (BAMF_TYPE_WINDOW, NULL);
-
+  BamfWindow *self = g_object_new (BAMF_TYPE_WINDOW, NULL);
   _bamf_view_set_path (BAMF_VIEW (self), path);
 
   return self;
