@@ -21,18 +21,17 @@
 #ifndef __BAMFAPPLICATION_H__
 #define __BAMFAPPLICATION_H__
 
-#include "bamf.h"
 #include "bamf-view.h"
 #include "bamf-window.h"
 #include <glib.h>
 #include <glib-object.h>
 
-#define BAMF_TYPE_APPLICATION			(bamf_application_get_type ())
-#define BAMF_APPLICATION(obj)			(G_TYPE_CHECK_INSTANCE_CAST ((obj), BAMF_TYPE_APPLICATION, BamfApplication))
-#define BAMF_IS_APPLICATION(obj)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), BAMF_TYPE_APPLICATION))
-#define BAMF_APPLICATION_CLASS(klass)		(G_TYPE_CHECK_CLASS_CAST ((klass), BAMF_TYPE_APPLICATION, BamfApplicationClass))
-#define BAMF_IS_APPLICATION_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE ((klass), BAMF_TYPE_APPLICATION))
-#define BAMF_APPLICATION_GET_CLASS(obj)		(G_TYPE_INSTANCE_GET_CLASS ((obj), BAMF_TYPE_APPLICATION, BamfApplicationClass))
+#define BAMF_TYPE_APPLICATION                   (bamf_application_get_type ())
+#define BAMF_APPLICATION(obj)                   (G_TYPE_CHECK_INSTANCE_CAST ((obj), BAMF_TYPE_APPLICATION, BamfApplication))
+#define BAMF_IS_APPLICATION(obj)                (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BAMF_TYPE_APPLICATION))
+#define BAMF_APPLICATION_CLASS(klass)           (G_TYPE_CHECK_CLASS_CAST ((klass), BAMF_TYPE_APPLICATION, BamfApplicationClass))
+#define BAMF_IS_APPLICATION_CLASS(klass)        (G_TYPE_CHECK_CLASS_TYPE ((klass), BAMF_TYPE_APPLICATION))
+#define BAMF_APPLICATION_GET_CLASS(obj)         (G_TYPE_INSTANCE_GET_CLASS ((obj), BAMF_TYPE_APPLICATION, BamfApplicationClass))
 
 typedef struct _BamfApplication BamfApplication;
 typedef struct _BamfApplicationClass BamfApplicationClass;
@@ -41,6 +40,12 @@ typedef struct _BamfApplicationPrivate BamfApplicationPrivate;
 struct _BamfApplicationClass
 {
   BamfViewClass parent;
+
+  void (*get_application_menu) (BamfApplication *application, gchar **name, gchar **path);
+  BamfView* (*get_focusable_child) (BamfApplication *application);
+  char ** (*get_supported_mime_types) (BamfApplication *application);
+  gboolean (*get_close_when_empty) (BamfApplication *application);
+  void (*supported_mimes_changed) (BamfApplication *application, const gchar **mimes);
 };
 
 struct _BamfApplication
@@ -51,20 +56,36 @@ struct _BamfApplication
   BamfApplicationPrivate *priv;
 };
 
+typedef enum
+{
+  BAMF_APPLICATION_SYSTEM,  /* BamfWindow container */
+  BAMF_APPLICATION_WEB,     /* BamfTab container */
+  BAMF_APPLICATION_UNKNOWN,
+} BamfApplicationType;
+
 GType             bamf_application_get_type                   (void) G_GNUC_CONST;
 
+void              bamf_application_emit_supported_mime_types_changed     (BamfApplication *application);
+
 const char      * bamf_application_get_desktop_file           (BamfApplication *application);
 void              bamf_application_set_desktop_file           (BamfApplication *application,
                                                                const char * desktop_file);
 
+char           ** bamf_application_get_supported_mime_types   (BamfApplication *application);
+
 GVariant        * bamf_application_get_xids                   (BamfApplication *application);
 
 gboolean          bamf_application_manages_xid                (BamfApplication *application,
                                                                guint32 xid);
 
-gboolean          bamf_application_contains_similar_to_window (BamfApplication *app, 
+BamfWindow      * bamf_application_get_window                 (BamfApplication *application,
+                                                               guint32 xid);
+
+gboolean          bamf_application_contains_similar_to_window (BamfApplication *app,
                                                                BamfWindow *window);
 
+gboolean          bamf_application_create_local_desktop_file  (BamfApplication *app);
+
 const char      * bamf_application_get_wmclass                (BamfApplication *application);
 void              bamf_application_set_wmclass                (BamfApplication *application,
                                                                const char *wmclass);
@@ -78,4 +99,20 @@ BamfApplication * bamf_application_new_from_desktop_files     (GList * desktop_f
 
 BamfApplication * bamf_application_new_with_wmclass           (const char *wmclass);
 
+void              bamf_application_set_application_type       (BamfApplication *application,
+                                                               BamfApplicationType type);
+BamfApplicationType bamf_application_get_application_type     (BamfApplication *application);
+
+void              bamf_application_get_application_menu       (BamfApplication *application,
+                                                               gchar **name, gchar **object_path);
+
+BamfView        * bamf_application_get_focusable_child        (BamfApplication *application);
+
+BamfView        * bamf_application_get_main_child             (BamfApplication *application);
+
+gboolean bamf_application_get_close_when_empty (BamfApplication *application);
+gboolean bamf_application_set_desktop_file_from_id (BamfApplication *application, const char *id);
+
+
+
 #endif
