@@ -22,7 +22,7 @@
 
 G_DEFINE_TYPE (BamfLegacyWindowTest, bamf_legacy_window_test, BAMF_TYPE_LEGACY_WINDOW);
 
-gint
+guint
 bamf_legacy_window_test_get_pid (BamfLegacyWindow *legacy_window)
 {
   BamfLegacyWindowTest *self;
@@ -127,16 +127,62 @@ bamf_legacy_window_test_is_skip_tasklist (BamfLegacyWindow *legacy_window)
 }
 
 void
-bamf_legacy_window_test_set_name (BamfLegacyWindowTest *self, char *val)
+bamf_legacy_window_test_set_name (BamfLegacyWindowTest *self, const char *val)
 {
   if (g_strcmp0 (self->name, val) == 0)
     return;
 
-  self->name = val;
+  g_free (self->name);
+  self->name = g_strdup (val);
 
   g_signal_emit_by_name (self, "name-changed");
 }
 
+void
+bamf_legacy_window_test_set_icon (BamfLegacyWindowTest *self, const char *val)
+{
+  if (g_strcmp0 (self->icon, val) == 0)
+    return;
+
+  g_free (self->icon);
+  self->icon = g_strdup (val);
+}
+
+void
+bamf_legacy_window_test_set_role (BamfLegacyWindowTest *self, const char *val)
+{
+  if (g_strcmp0 (self->role, val) == 0)
+    return;
+
+  g_free (self->role);
+  self->role = g_strdup (val);
+
+  g_signal_emit_by_name (self, "role-changed");
+}
+
+void
+bamf_legacy_window_test_set_wmclass (BamfLegacyWindowTest *self, const char *class_name, const char *instance_name)
+{
+  gboolean changed = FALSE;
+
+  if (g_strcmp0 (self->wm_class_name, class_name) != 0)
+    {
+      g_free (self->wm_class_name);
+      self->wm_class_name = g_strdup (class_name);
+      changed = TRUE;
+    }
+
+  if (g_strcmp0 (self->wm_class_instance, instance_name) != 0)
+    {
+      g_free (self->wm_class_instance);
+      self->wm_class_instance = g_strdup (instance_name);
+      changed = TRUE;
+    }
+
+  if (changed)
+    g_signal_emit_by_name (self, "class-changed");
+}
+
 static const char *
 bamf_legacy_window_test_get_name (BamfLegacyWindow *legacy_window)
 {
@@ -144,7 +190,25 @@ bamf_legacy_window_test_get_name (BamfLegacyWindow *legacy_window)
 
   self = BAMF_LEGACY_WINDOW_TEST (legacy_window);
 
-  return  self->name;
+  return self->name;
+}
+
+static const char *
+bamf_legacy_window_test_get_role (BamfLegacyWindow *legacy_window)
+{
+  BamfLegacyWindowTest *self;
+
+  self = BAMF_LEGACY_WINDOW_TEST (legacy_window);
+
+  return self->role;
+}
+
+static BamfLegacyWindow *
+bamf_legacy_window_test_get_transient (BamfLegacyWindow *legacy_window)
+{
+  BamfLegacyWindowTest *self = BAMF_LEGACY_WINDOW_TEST (legacy_window);
+
+  return self->transient_window;
 }
 
 static const char *
@@ -154,7 +218,7 @@ bamf_legacy_window_test_get_class_name (BamfLegacyWindow *legacy_window)
 
   self = BAMF_LEGACY_WINDOW_TEST (legacy_window);
 
-  return  self->wm_class_name;
+  return self->wm_class_name;
 }
 
 static const char *
@@ -164,10 +228,10 @@ bamf_legacy_window_test_get_class_instance_name (BamfLegacyWindow *legacy_window
 
   self = BAMF_LEGACY_WINDOW_TEST (legacy_window);
 
-  return  self->wm_class_instance;
+  return self->wm_class_instance;
 }
 
-char *
+const char *
 bamf_legacy_window_test_get_exec_string (BamfLegacyWindow *legacy_window)
 {
   BamfLegacyWindowTest *self;
@@ -177,6 +241,26 @@ bamf_legacy_window_test_get_exec_string (BamfLegacyWindow *legacy_window)
   return self->exec;
 }
 
+const char *
+bamf_legacy_window_test_get_working_dir (BamfLegacyWindow *legacy_window)
+{
+  BamfLegacyWindowTest *self;
+
+  self = BAMF_LEGACY_WINDOW_TEST (legacy_window);
+
+  return self->working_dir;
+}
+
+char *
+bamf_legacy_window_test_get_process_name (BamfLegacyWindow *legacy_window)
+{
+  BamfLegacyWindowTest *self;
+
+  self = BAMF_LEGACY_WINDOW_TEST (legacy_window);
+
+  return g_strdup (self->process_name);
+}
+
 char *
 bamf_legacy_window_test_get_app_id (BamfLegacyWindow *legacy_window)
 {
@@ -185,7 +269,7 @@ bamf_legacy_window_test_get_app_id (BamfLegacyWindow *legacy_window)
 
   self = BAMF_LEGACY_WINDOW_TEST (legacy_window);
 
-  return self->application_id;
+  return g_strdup (self->application_id);
 }
 
 char *
@@ -196,7 +280,7 @@ bamf_legacy_window_test_get_unique_bus_name (BamfLegacyWindow *legacy_window)
 
   self = BAMF_LEGACY_WINDOW_TEST (legacy_window);
 
-  return self->unique_bus_name;
+  return g_strdup (self->unique_bus_name);
 }
 
 char *
@@ -207,7 +291,7 @@ bamf_legacy_window_test_get_menu_object_path (BamfLegacyWindow *legacy_window)
 
   self = BAMF_LEGACY_WINDOW_TEST (legacy_window);
 
-  return self->dbus_menu_object_path;
+  return g_strdup (self->dbus_menu_object_path);
 }
 
 void
@@ -328,10 +412,49 @@ bamf_legacy_window_test_get_window_type (BamfLegacyWindow *window)
   return self->window_type;
 }
 
-void
-bamf_legacy_window_test_dispose (GObject *object)
+char *
+bamf_legacy_window_test_get_hint (BamfLegacyWindow *window, const char *name)
 {
-  G_OBJECT_CLASS (bamf_legacy_window_test_parent_class)->dispose (object);
+  g_return_val_if_fail (BAMF_IS_LEGACY_WINDOW_TEST (window), NULL);
+  BamfLegacyWindowTest *self = BAMF_LEGACY_WINDOW_TEST (window);
+
+  return g_strdup (g_hash_table_lookup (self->hints, name));
+}
+
+void
+bamf_legacy_window_test_set_hint (BamfLegacyWindow *window, const char *name, const char *value)
+{
+  g_return_if_fail (BAMF_IS_LEGACY_WINDOW_TEST (window));
+  BamfLegacyWindowTest *self = BAMF_LEGACY_WINDOW_TEST (window);
+
+  return g_hash_table_insert (self->hints, g_strdup (name), g_strdup (value));
+}
+
+static char *
+bamf_legacy_window_test_save_mini_icon (BamfLegacyWindow *window)
+{
+  return g_strdup (BAMF_LEGACY_WINDOW_TEST (window)->icon);
+}
+
+static void
+bamf_legacy_window_test_finalize (GObject *object)
+{
+  BamfLegacyWindowTest *self = BAMF_LEGACY_WINDOW_TEST (object);
+
+  g_free (self->name);
+  g_free (self->icon);
+  g_free (self->role);
+  g_free (self->wm_class_name);
+  g_free (self->wm_class_instance);
+  g_free (self->exec);
+  g_free (self->working_dir);
+  g_free (self->process_name);
+  g_free (self->application_id);
+  g_free (self->unique_bus_name);
+  g_free (self->dbus_menu_object_path);
+  g_hash_table_unref (self->hints);
+
+  G_OBJECT_CLASS (bamf_legacy_window_test_parent_class)->finalize (object);
 }
 
 void
@@ -340,11 +463,16 @@ bamf_legacy_window_test_class_init (BamfLegacyWindowTestClass *klass)
   BamfLegacyWindowClass *win_class = BAMF_LEGACY_WINDOW_CLASS (klass);
   GObjectClass *obj_class = G_OBJECT_CLASS (klass);
 
-  obj_class->dispose          = bamf_legacy_window_test_dispose;
+  obj_class->finalize         = bamf_legacy_window_test_finalize;
+  win_class->get_transient    = bamf_legacy_window_test_get_transient;
   win_class->get_name         = bamf_legacy_window_test_get_name;
+  win_class->save_mini_icon   = bamf_legacy_window_test_save_mini_icon;
+  win_class->get_role         = bamf_legacy_window_test_get_role;
   win_class->get_class_name   = bamf_legacy_window_test_get_class_name;
   win_class->get_class_instance_name = bamf_legacy_window_test_get_class_instance_name;
   win_class->get_exec_string  = bamf_legacy_window_test_get_exec_string;
+  win_class->get_working_dir  = bamf_legacy_window_test_get_working_dir;
+  win_class->get_process_name = bamf_legacy_window_test_get_process_name;
   win_class->get_xid          = bamf_legacy_window_test_get_xid;
   win_class->get_pid          = bamf_legacy_window_test_get_pid;
   win_class->needs_attention  = bamf_legacy_window_test_needs_attention;
@@ -358,6 +486,8 @@ bamf_legacy_window_test_class_init (BamfLegacyWindowTestClass *klass)
   win_class->get_window_type  = bamf_legacy_window_test_get_window_type;
   win_class->maximized        = bamf_legacy_window_test_maximized;
   win_class->is_closed        = bamf_legacy_window_test_is_closed;
+  win_class->get_hint         = bamf_legacy_window_test_get_hint;
+  win_class->set_hint         = bamf_legacy_window_test_set_hint;
   win_class->reopen           = bamf_legacy_window_test_reopen;
 }
 
@@ -368,6 +498,7 @@ bamf_legacy_window_test_init (BamfLegacyWindowTest *self)
   self->pid = g_random_int_range (1, 100000);
   self->maximized = BAMF_WINDOW_FLOATING;
   self->is_closed = FALSE;
+  self->hints = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 }
 
 
@@ -380,12 +511,17 @@ bamf_legacy_window_copy (BamfLegacyWindowTest *self)
   copy->xid = self->xid;
   copy->pid = self->pid;
   copy->name = g_strdup (self->name);
+  copy->icon = g_strdup (self->icon);
+  copy->role = g_strdup (self->role);
   copy->wm_class_name = g_strdup (self->wm_class_name);
   copy->wm_class_instance = g_strdup (self->wm_class_instance);
   copy->exec = g_strdup (self->exec);
+  copy->working_dir = g_strdup (self->working_dir);
+  copy->process_name = g_strdup (self->process_name);
   copy->application_id = g_strdup (self->application_id);
   copy->unique_bus_name = g_strdup (self->unique_bus_name);
   copy->dbus_menu_object_path = g_strdup (self->dbus_menu_object_path);
+  copy->transient_window = self->transient_window;
   copy->needs_attention = self->needs_attention;
   copy->is_desktop = self->is_desktop;
   copy->is_skip = self->is_skip;
@@ -394,12 +530,13 @@ bamf_legacy_window_copy (BamfLegacyWindowTest *self)
   copy->geometry = self->geometry;
   copy->maximized = self->maximized;
   copy->window_type = self->window_type;
+  copy->hints = g_hash_table_ref (self->hints);
 
   return copy;
 }
 
 BamfLegacyWindowTest *
-bamf_legacy_window_test_new (guint32 xid, gchar *name, gchar *wmclass_name, gchar *exec)
+bamf_legacy_window_test_new (guint32 xid, const gchar *name, const gchar *wmclass_name, const gchar *exec)
 {
   BamfLegacyWindowTest *self;
 
@@ -409,6 +546,15 @@ bamf_legacy_window_test_new (guint32 xid, gchar *name, gchar *wmclass_name, gcha
   self->name = g_strdup (name);
   self->wm_class_name = g_strdup (wmclass_name);
   self->exec = g_strdup (exec);
+  self->working_dir = g_get_current_dir ();
+
+  if (self->exec)
+    {
+      gchar **splitted_exec = g_strsplit (exec, " ", 2);
+      gchar *tmp = g_utf8_strrchr (splitted_exec[0], -1, G_DIR_SEPARATOR);
+      self->process_name = g_strdup (tmp ? tmp + 1 : splitted_exec[0]);
+      g_strfreev (splitted_exec);
+    }
 
   return self;
 }
