@@ -24,6 +24,7 @@
 #include "bamf-legacy-screen.h"
 #include "bamf-xutils.h"
 #include <libgtop-2.0/glibtop.h>
+#include <libgtop-2.0/glibtop/procwd.h>
 #include <glibtop/procargs.h>
 #include <glibtop/procuid.h>
 #include <stdio.h>
@@ -32,9 +33,13 @@ G_DEFINE_TYPE (BamfLegacyWindow, bamf_legacy_window, G_TYPE_OBJECT);
 #define BAMF_LEGACY_WINDOW_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE(obj, \
 BAMF_TYPE_LEGACY_WINDOW, BamfLegacyWindowPrivate))
 
+#define WNCK_WINDOW_BAMF_DATA "bamf-legacy-window"
+
 enum
 {
   NAME_CHANGED,
+  ROLE_CHANGED,
+  CLASS_CHANGED,
   STATE_CHANGED,
   GEOMETRY_CHANGED,
   CLOSED,
@@ -47,15 +52,9 @@ static guint legacy_window_signals[LAST_SIGNAL] = { 0 };
 struct _BamfLegacyWindowPrivate
 {
   WnckWindow * legacy_window;
-  char       * mini_icon_path;
-#ifndef USE_GTK3
-  char       * group_name;
-  char       * instance_name;
-#endif
-  gulong       closed_id;
-  gulong       name_changed_id;
-  gulong       state_changed_id;
-  gulong       geometry_changed_id;
+  GFile      * mini_icon;
+  gchar      * exec_string;
+  gchar      * working_dir;
   gboolean     is_closed;
 };
 
@@ -128,18 +127,7 @@ bamf_legacy_window_get_class_instance_name (BamfLegacyWindow *self)
   if (!window)
     return NULL;
 
-#ifdef USE_GTK3
   return wnck_window_get_class_instance_name (window);
-
-#else
-  if (!self->priv->instance_name)
-    {
-      Window xid = wnck_window_get_xid (window);
-      bamf_xutils_get_window_class_hints (xid, &self->priv->instance_name, NULL);
-    }
-
-  return self->priv->instance_name;
-#endif
 }
 
 const char *
@@ -157,18 +145,7 @@ bamf_legacy_window_get_class_name (BamfLegacyWindow *self)
   if (!window)
     return NULL;
 
-#ifdef USE_GTK3
   return wnck_window_get_class_group_name (window);
-
-#else
-  if (!self->priv->group_name)
-    {
-      Window xid = wnck_window_get_xid (window);
-      bamf_xutils_get_window_class_hints (xid, NULL, &self->priv->group_name);
-    }
-
-  return self->priv->group_name;
-#endif
 }
 
 const char *
@@ -176,22 +153,79 @@ bamf_legacy_window_get_name (BamfLegacyWindow *self)
 {
   g_return_val_if_fail (BAMF_IS_LEGACY_WINDOW (self), NULL);
 
-
   if (BAMF_LEGACY_WINDOW_GET_CLASS (self)->get_name)
     return BAMF_LEGACY_WINDOW_GET_CLASS (self)->get_name (self);
 
   if (!self->priv->legacy_window)
     return NULL;
+
   return wnck_window_get_name (self->priv->legacy_window);
 }
 
+const char *
+bamf_legacy_window_get_role (BamfLegacyWindow *self)
+{
+  g_return_val_if_fail (BAMF_IS_LEGACY_WINDOW (self), NULL);
+
+  if (BAMF_LEGACY_WINDOW_GET_CLASS (self)->get_role)
+    return BAMF_LEGACY_WINDOW_GET_CLASS (self)->get_role (self);
+
+  if (!self->priv->legacy_window)
+    return NULL;
+
+  return wnck_window_get_role (self->priv->legacy_window);
+}
+
 char *
+bamf_legacy_window_get_process_name (BamfLegacyWindow *self)
+{
+  gchar *stat_path;
+  gchar *contents;
+  gchar **lines;
+  gchar **sections;
+  gchar *result = NULL;
+  guint pid;
+
+  g_return_val_if_fail (BAMF_IS_LEGACY_WINDOW (self), NULL);
+
+  if (BAMF_LEGACY_WINDOW_GET_CLASS (self)->get_process_name)
+    return BAMF_LEGACY_WINDOW_GET_CLASS (self)->get_process_name (self);
+
+  pid = bamf_legacy_window_get_pid (self);
+
+  if (pid <= 0)
+    return NULL;
+
+  stat_path = g_strdup_printf ("/proc/%i/status", pid);
+
+  if (g_file_get_contents (stat_path, &contents, NULL, NULL))
+    {
+      lines = g_strsplit (contents, "\n", 2);
+
+      if (lines && g_strv_length (lines) > 0)
+        {
+          sections = g_strsplit (lines[0], "\t", 0);
+          if (sections)
+            {
+              if (g_strv_length (sections) > 1)
+                result = g_strdup (sections[1]);
+
+              g_strfreev (sections);
+            }
+          g_strfreev (lines);
+        }
+      g_free (contents);
+    }
+  g_free (stat_path);
+
+  return result;
+}
+
+const char *
 bamf_legacy_window_get_exec_string (BamfLegacyWindow *self)
 {
-  gchar *result = NULL;
-  gint pid = 0, i = 0;
-  gchar **argv = NULL;
-  GString *exec = NULL;
+  guint pid;
+  gchar **argv;
   glibtop_proc_args buffer;
 
   g_return_val_if_fail (BAMF_IS_LEGACY_WINDOW (self), NULL);
@@ -199,68 +233,115 @@ bamf_legacy_window_get_exec_string (BamfLegacyWindow *self)
   if (BAMF_LEGACY_WINDOW_GET_CLASS (self)->get_exec_string)
     return BAMF_LEGACY_WINDOW_GET_CLASS (self)->get_exec_string (self);
 
+  if (self->priv->exec_string)
+    return self->priv->exec_string;
+
   pid = bamf_legacy_window_get_pid (self);
 
   if (pid == 0)
     return NULL;
 
   argv = glibtop_get_proc_argv (&buffer, pid, 0);
-  exec = g_string_new ("");
+  self->priv->exec_string = g_strstrip (g_strjoinv (" ", argv));
+  g_strfreev (argv);
 
-  while (argv[i] != NULL)
-    {
-      g_string_append (exec, argv[i]);
-      if (argv[i + 1] != NULL)
-        g_string_append (exec, " ");
-      g_free (argv[i]);
-      i++;
-    }
-
-  g_free (argv);
-
-  result = g_strdup (exec->str);
-  g_string_free (exec, TRUE);
-  return result;
+  return self->priv->exec_string;
 }
 
 const char *
+bamf_legacy_window_get_working_dir (BamfLegacyWindow *self)
+{
+  guint pid = 0;
+  gchar **dirs;
+  glibtop_proc_wd buffer_wd;
+
+  g_return_val_if_fail (BAMF_IS_LEGACY_WINDOW (self), NULL);
+
+  if (BAMF_LEGACY_WINDOW_GET_CLASS (self)->get_working_dir)
+    return BAMF_LEGACY_WINDOW_GET_CLASS (self)->get_working_dir (self);
+
+  if (self->priv->working_dir)
+    return self->priv->working_dir;
+
+  pid = bamf_legacy_window_get_pid (self);
+
+  if (pid == 0)
+    return NULL;
+
+  dirs = glibtop_get_proc_wd (&buffer_wd, pid);
+
+  if (!dirs)
+    return NULL;
+
+  self->priv->working_dir = g_strdup (dirs[0] ? g_strstrip (dirs[0]) : NULL);
+  g_strfreev (dirs);
+
+  return self->priv->working_dir;
+}
+
+char *
 bamf_legacy_window_save_mini_icon (BamfLegacyWindow *self)
 {
   WnckWindow *window;
   GdkPixbuf *pbuf;
-  char *tmp;
-  
+  GFile *tmp;
+  GFileIOStream *iostream;
+  GOutputStream *output;
+
   g_return_val_if_fail (BAMF_IS_LEGACY_WINDOW (self), NULL);
-  
-  if (self->priv->mini_icon_path)
+
+  if (BAMF_LEGACY_WINDOW_GET_CLASS (self)->save_mini_icon)
+    return BAMF_LEGACY_WINDOW_GET_CLASS (self)->save_mini_icon (self);
+
+  if (self->priv->mini_icon)
     {
-      if (g_file_test (self->priv->mini_icon_path, G_FILE_TEST_EXISTS))
-        return self->priv->mini_icon_path;
+      if (g_file_query_exists (self->priv->mini_icon, NULL))
+        {
+          return g_file_get_path (self->priv->mini_icon);
+        }
       else
-        g_free (self->priv->mini_icon_path);
+        {
+          g_object_unref (self->priv->mini_icon);
+          self->priv->mini_icon = NULL;
+        }
     }
-  
+
   window = self->priv->legacy_window;
-  
+
   if (!window)
     return NULL;
-  
+
   if (wnck_window_get_icon_is_fallback (window))
     return NULL;
-  
-  tmp = tmpnam (NULL);
+
+  tmp = g_file_new_tmp (".bamficonXXXXXX", &iostream, NULL);
+
   if (!tmp)
     return NULL;
-  
+
+  output = g_io_stream_get_output_stream (G_IO_STREAM (iostream));
   pbuf = wnck_window_get_icon (window);
-  if (!gdk_pixbuf_save (pbuf, tmp, "png", NULL, NULL))
-    return NULL;
-  
-  self->priv->mini_icon_path = g_strdup (tmp);
-  return self->priv->mini_icon_path;
+
+  if (gdk_pixbuf_save_to_stream (pbuf, output, "png", NULL, NULL, NULL))
+    {
+      self->priv->mini_icon = g_object_ref (tmp);
+    }
+
+  g_object_unref (iostream);
+  g_object_unref (tmp);
+
+  return g_file_get_path (self->priv->mini_icon);
 }
 
-gint
+GFile *
+bamf_legacy_window_get_saved_mini_icon (BamfLegacyWindow *self)
+{
+  g_return_val_if_fail (BAMF_IS_LEGACY_WINDOW (self), NULL);
+
+  return self->priv->mini_icon;
+}
+
+guint
 bamf_legacy_window_get_pid (BamfLegacyWindow *self)
 {
   g_return_val_if_fail (BAMF_IS_LEGACY_WINDOW (self), 0);
@@ -270,7 +351,9 @@ bamf_legacy_window_get_pid (BamfLegacyWindow *self)
 
   if (!self->priv->legacy_window)
     return 0;
-  return wnck_window_get_pid (self->priv->legacy_window);
+
+  int pid = wnck_window_get_pid (self->priv->legacy_window);
+  return G_LIKELY (pid >= 0) ? pid : 0;
 }
 
 guint32
@@ -294,28 +377,30 @@ bamf_legacy_window_get_transient (BamfLegacyWindow *self)
   BamfLegacyWindow *other;
   GList *windows, *l;
   WnckWindow *transient_legacy;
-  
+
   g_return_val_if_fail (BAMF_IS_LEGACY_WINDOW (self), NULL);
-  
+
+  if (BAMF_LEGACY_WINDOW_GET_CLASS (self)->get_transient)
+    return BAMF_LEGACY_WINDOW_GET_CLASS (self)->get_transient (self);
+
+  g_return_val_if_fail (self->priv->legacy_window, NULL);
+
   transient_legacy = wnck_window_get_transient (self->priv->legacy_window);
-  if (transient_legacy == NULL)
-    return NULL;
-  
   screen = bamf_legacy_screen_get_default ();
   g_return_val_if_fail (BAMF_IS_LEGACY_SCREEN (screen), NULL);
-  
+
   windows = bamf_legacy_screen_get_windows (screen);
   for (l = windows; l; l = l->next)
     {
       other = l->data;
-      
+
       if (!BAMF_IS_LEGACY_WINDOW (other))
         continue;
-      
+
       if (other->priv->legacy_window == transient_legacy)
         return other;
     }
-  
+
   return NULL;
 }
 
@@ -323,21 +408,22 @@ gint
 bamf_legacy_window_get_stacking_position (BamfLegacyWindow *self)
 {
   BamfLegacyScreen *screen;
-  
+
   g_return_val_if_fail (BAMF_IS_LEGACY_WINDOW (self), -1);
 
   screen = bamf_legacy_screen_get_default ();
   g_return_val_if_fail (BAMF_IS_LEGACY_SCREEN (screen), -1);
-  
+
   return g_list_index (bamf_legacy_screen_get_windows (screen), self);
 }
 
 static void
-handle_name_changed (WnckWindow *window, BamfLegacyWindow *self)
+handle_window_signal (WnckWindow *window, gpointer data)
 {
+  BamfLegacyWindow *self = g_object_get_data (G_OBJECT (window), WNCK_WINDOW_BAMF_DATA);
   g_return_if_fail (BAMF_IS_LEGACY_WINDOW (self));
 
-  g_signal_emit (self, legacy_window_signals[NAME_CHANGED], 0);
+  g_signal_emit (self, legacy_window_signals[GPOINTER_TO_UINT (data)], 0);
 }
 
 static void
@@ -351,14 +437,6 @@ handle_state_changed (WnckWindow *window,
   g_signal_emit (self, legacy_window_signals[STATE_CHANGED], 0);
 }
 
-static void
-handle_geometry_changed (WnckWindow *window, BamfLegacyWindow *self)
-{
-  g_return_if_fail (BAMF_IS_LEGACY_WINDOW (self));
-
-  g_signal_emit (self, legacy_window_signals[GEOMETRY_CHANGED], 0);
-}
-
 gboolean
 bamf_legacy_window_is_closed (BamfLegacyWindow *self)
 {
@@ -460,6 +538,36 @@ bamf_legacy_window_set_hint (BamfLegacyWindow *self, const char *name, const cha
   bamf_xutils_set_string_window_hint (xid, name, value);
 }
 
+static void
+top_window_action_menu (GtkMenu *menu, gint *x, gint *y, gboolean *push, gpointer data)
+{
+  BamfLegacyWindow *self = data;
+  gint w, h;
+  wnck_window_get_client_window_geometry (self->priv->legacy_window, x, y, &w, &h);
+  *push = TRUE;
+}
+
+void bamf_legacy_window_show_action_menu (BamfLegacyWindow *self, guint32 time,
+                                          guint button, gint x, gint y)
+{
+  g_return_if_fail (BAMF_IS_LEGACY_WINDOW (self));
+
+  if (BAMF_LEGACY_WINDOW_GET_CLASS (self)->show_action_menu)
+    return BAMF_LEGACY_WINDOW_GET_CLASS (self)->show_action_menu (self, button, time, x, y);
+
+  g_return_if_fail (WNCK_IS_WINDOW (self->priv->legacy_window));
+
+  GtkWidget *menu = wnck_action_menu_new (self->priv->legacy_window);
+  g_signal_connect (G_OBJECT (menu), "unmap", G_CALLBACK (g_object_unref), NULL);
+  g_object_ref_sink (menu);
+
+  gtk_menu_set_screen (GTK_MENU (menu), gdk_screen_get_default ());
+  gtk_widget_show (menu);
+
+  GtkMenuPositionFunc position = button ? NULL : top_window_action_menu;
+  gtk_menu_popup (GTK_MENU (menu), NULL, NULL, position, self, button, time);
+}
+
 static void
 handle_window_closed (WnckScreen *screen,
                       WnckWindow *window,
@@ -512,47 +620,34 @@ static void
 bamf_legacy_window_dispose (GObject *object)
 {
   BamfLegacyWindow *self;
-  GFile *file;
+  guint i;
 
   self = BAMF_LEGACY_WINDOW (object);
 
-  g_signal_handler_disconnect (wnck_screen_get_default (),
-                               self->priv->closed_id);
-
-  if (self->priv->mini_icon_path)
+  if (self->priv->mini_icon)
     {
-      file = g_file_new_for_path (self->priv->mini_icon_path);
-      g_file_delete (file, NULL, NULL);
-      g_object_unref (file);
-
-      g_free (self->priv->mini_icon_path);
-      self->priv->mini_icon_path = NULL;
+      g_file_delete (self->priv->mini_icon, NULL, NULL);
+      g_clear_object (&self->priv->mini_icon);
     }
 
-#ifndef USE_GTK3
-  if (self->priv->group_name)
-    {
-      g_free (self->priv->group_name);
-      self->priv->group_name = NULL;
-    }
+  g_clear_pointer (&self->priv->exec_string, g_free);
+  g_clear_pointer (&self->priv->working_dir, g_free);
 
-  if (self->priv->instance_name)
-    {
-      g_free (self->priv->instance_name);
-      self->priv->instance_name = NULL;
-    }
-#endif
+  g_signal_handlers_disconnect_by_data (wnck_screen_get_default (), self);
 
   if (self->priv->legacy_window)
     {
-      g_signal_handler_disconnect (self->priv->legacy_window,
-                                   self->priv->name_changed_id);
+      g_object_set_data (G_OBJECT (self->priv->legacy_window), WNCK_WINDOW_BAMF_DATA, NULL);
+      g_signal_handlers_disconnect_by_data (self->priv->legacy_window, self);
 
-      g_signal_handler_disconnect (self->priv->legacy_window,
-                                   self->priv->state_changed_id);
+      for (i = 0; i < LAST_SIGNAL; ++i)
+        {
+          g_signal_handlers_disconnect_by_func (self->priv->legacy_window,
+                                                handle_window_signal,
+                                                GUINT_TO_POINTER (NAME_CHANGED));
+        }
 
-      g_signal_handler_disconnect (self->priv->legacy_window,
-                                   self->priv->geometry_changed_id);
+      self->priv->legacy_window = NULL;
     }
 
   G_OBJECT_CLASS (bamf_legacy_window_parent_class)->dispose (object);
@@ -561,15 +656,10 @@ bamf_legacy_window_dispose (GObject *object)
 static void
 bamf_legacy_window_init (BamfLegacyWindow * self)
 {
-  WnckScreen *screen;
+  self->priv = BAMF_LEGACY_WINDOW_GET_PRIVATE (self);
 
-  BamfLegacyWindowPrivate *priv;
-  priv = self->priv = BAMF_LEGACY_WINDOW_GET_PRIVATE (self);
-
-  screen = wnck_screen_get_default ();
-
-  priv->closed_id = g_signal_connect (G_OBJECT (screen), "window-closed",
-                                      (GCallback) handle_window_closed, self);
+  g_signal_connect (wnck_screen_get_default (), "window-closed",
+                    (GCallback) handle_window_closed, self);
 }
 
 static void
@@ -577,7 +667,7 @@ bamf_legacy_window_class_init (BamfLegacyWindowClass * klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-  object_class->dispose      = bamf_legacy_window_dispose;
+  object_class->dispose = bamf_legacy_window_dispose;
 
   g_type_class_add_private (klass, sizeof (BamfLegacyWindowPrivate));
 
@@ -586,36 +676,42 @@ bamf_legacy_window_class_init (BamfLegacyWindowClass * klass)
                   G_OBJECT_CLASS_TYPE (klass),
                   G_SIGNAL_RUN_FIRST,
                   G_STRUCT_OFFSET (BamfLegacyWindowClass, name_changed),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
-                  G_TYPE_NONE, 0);
+                  NULL, NULL, NULL, G_TYPE_NONE, 0);
+
+  legacy_window_signals [ROLE_CHANGED] =
+    g_signal_new (BAMF_LEGACY_WINDOW_SIGNAL_ROLE_CHANGED,
+                  G_OBJECT_CLASS_TYPE (klass),
+                  G_SIGNAL_RUN_FIRST,
+                  G_STRUCT_OFFSET (BamfLegacyWindowClass, role_changed),
+                  NULL, NULL, NULL, G_TYPE_NONE, 0);
+
+  legacy_window_signals [CLASS_CHANGED] =
+    g_signal_new (BAMF_LEGACY_WINDOW_SIGNAL_CLASS_CHANGED,
+                  G_OBJECT_CLASS_TYPE (klass),
+                  G_SIGNAL_RUN_FIRST,
+                  G_STRUCT_OFFSET (BamfLegacyWindowClass, class_changed),
+                  NULL, NULL, NULL, G_TYPE_NONE, 0);
 
   legacy_window_signals [STATE_CHANGED] =
     g_signal_new (BAMF_LEGACY_WINDOW_SIGNAL_STATE_CHANGED,
                   G_OBJECT_CLASS_TYPE (klass),
                   G_SIGNAL_RUN_FIRST,
                   G_STRUCT_OFFSET (BamfLegacyWindowClass, state_changed),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
-                  G_TYPE_NONE, 0);
+                  NULL, NULL, NULL, G_TYPE_NONE, 0);
 
   legacy_window_signals [GEOMETRY_CHANGED] =
     g_signal_new (BAMF_LEGACY_WINDOW_SIGNAL_GEOMETRY_CHANGED,
                   G_OBJECT_CLASS_TYPE (klass),
                   G_SIGNAL_RUN_FIRST,
                   G_STRUCT_OFFSET (BamfLegacyWindowClass, geometry_changed),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
-                  G_TYPE_NONE, 0);
+                  NULL, NULL, NULL, G_TYPE_NONE, 0);
 
   legacy_window_signals [CLOSED] =
     g_signal_new (BAMF_LEGACY_WINDOW_SIGNAL_CLOSED,
                   G_OBJECT_CLASS_TYPE (klass),
                   G_SIGNAL_RUN_FIRST,
                   G_STRUCT_OFFSET (BamfLegacyWindowClass, closed),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
-                  G_TYPE_NONE, 0);
+                  NULL, NULL, NULL, G_TYPE_NONE, 0);
 }
 
 BamfLegacyWindow *
@@ -626,14 +722,29 @@ bamf_legacy_window_new (WnckWindow *legacy_window)
 
   self->priv->legacy_window = legacy_window;
 
-  self->priv->name_changed_id = g_signal_connect (G_OBJECT (legacy_window), "name-changed",
-                                                  (GCallback) handle_name_changed, self);
+  g_return_val_if_fail (WNCK_IS_WINDOW (self->priv->legacy_window), self);
+  g_warn_if_fail (!g_object_get_data (G_OBJECT (legacy_window), WNCK_WINDOW_BAMF_DATA));
 
-  self->priv->state_changed_id = g_signal_connect (G_OBJECT (legacy_window), "state-changed",
-                                                   (GCallback) handle_state_changed, self);
+  g_object_set_data (G_OBJECT (legacy_window), WNCK_WINDOW_BAMF_DATA, self);
 
-  self->priv->geometry_changed_id = g_signal_connect (G_OBJECT (legacy_window), "geometry-changed",
-                                                      (GCallback) handle_geometry_changed, self);
+  g_signal_connect (G_OBJECT (legacy_window), "name-changed",
+                    G_CALLBACK (handle_window_signal),
+                    GUINT_TO_POINTER (NAME_CHANGED));
+
+  g_signal_connect (G_OBJECT (legacy_window), "role-changed",
+                    G_CALLBACK (handle_window_signal),
+                    GUINT_TO_POINTER (ROLE_CHANGED));
+
+  g_signal_connect (G_OBJECT (legacy_window), "class-changed",
+                    G_CALLBACK (handle_window_signal),
+                    GUINT_TO_POINTER (CLASS_CHANGED));
+
+  g_signal_connect (G_OBJECT (legacy_window), "geometry-changed",
+                    G_CALLBACK (handle_window_signal),
+                    GUINT_TO_POINTER (GEOMETRY_CHANGED));
+
+  g_signal_connect (G_OBJECT (legacy_window), "state-changed",
+                    G_CALLBACK (handle_state_changed), self);
 
   return self;
 }
