@@ -23,7 +23,6 @@
 
 #include "config.h"
 
-#include "bamf.h"
 #include "bamf-view.h"
 #include <sys/types.h>
 #include <glib.h>
@@ -38,6 +37,8 @@
 #define BAMF_LEGACY_WINDOW_GET_CLASS(obj)       (G_TYPE_INSTANCE_GET_CLASS ((obj), BAMF_TYPE_LEGACY_WINDOW, BamfLegacyWindowClass))
 
 #define BAMF_LEGACY_WINDOW_SIGNAL_NAME_CHANGED     "name-changed"
+#define BAMF_LEGACY_WINDOW_SIGNAL_ROLE_CHANGED     "role-changed"
+#define BAMF_LEGACY_WINDOW_SIGNAL_CLASS_CHANGED    "class-changed"
 #define BAMF_LEGACY_WINDOW_SIGNAL_STATE_CHANGED    "state-changed"
 #define BAMF_LEGACY_WINDOW_SIGNAL_GEOMETRY_CHANGED "geometry-changed"
 #define BAMF_LEGACY_WINDOW_SIGNAL_CLOSED           "closed"
@@ -70,16 +71,21 @@ struct _BamfLegacyWindowClass
 {
   GObjectClass parent;
 
+  BamfLegacyWindow * (*get_transient)     (BamfLegacyWindow *legacy_window);
   const char * (*get_name)                (BamfLegacyWindow *legacy_window);
+  const char * (*get_role)                (BamfLegacyWindow *legacy_window);
   const char * (*get_class_name)          (BamfLegacyWindow *legacy_window);
   const char * (*get_class_instance_name) (BamfLegacyWindow *legacy_window);
-  char       * (*get_exec_string)         (BamfLegacyWindow *legacy_window);
+  const char * (*get_exec_string)         (BamfLegacyWindow *legacy_window);
+  const char * (*get_working_dir)         (BamfLegacyWindow *legacy_window);
+  char       * (*save_mini_icon)          (BamfLegacyWindow *legacy_window);
+  char       * (*get_process_name)        (BamfLegacyWindow *legacy_window);
   char       * (*get_app_id)              (BamfLegacyWindow *legacy_window);
   char       * (*get_unique_bus_name)     (BamfLegacyWindow *legacy_window);
   char       * (*get_menu_object_path)    (BamfLegacyWindow *legacy_window);
   char       * (*get_hint)                (BamfLegacyWindow *legacy_window,
                                            const gchar *name);
-  gint         (*get_pid)                 (BamfLegacyWindow *legacy_window);
+  guint        (*get_pid)                 (BamfLegacyWindow *legacy_window);
   guint32      (*get_xid)                 (BamfLegacyWindow *legacy_window);
   gboolean     (*needs_attention)         (BamfLegacyWindow *legacy_window);
   gboolean     (*is_active)               (BamfLegacyWindow *legacy_window);
@@ -97,6 +103,8 @@ struct _BamfLegacyWindowClass
 
   /*< signals >*/
   void     (*name_changed)     (BamfLegacyWindow *legacy_window);
+  void     (*class_changed)    (BamfLegacyWindow *legacy_window);
+  void     (*role_changed)     (BamfLegacyWindow *legacy_window);
   void     (*state_changed)    (BamfLegacyWindow *legacy_window);
   void     (*geometry_changed) (BamfLegacyWindow *legacy_window);
   void     (*closed)           (BamfLegacyWindow *legacy_window);
@@ -114,7 +122,7 @@ GType              bamf_legacy_window_get_type             (void) G_GNUC_CONST;
 
 guint32            bamf_legacy_window_get_xid              (BamfLegacyWindow *self);
 
-gint               bamf_legacy_window_get_pid              (BamfLegacyWindow *self);
+guint              bamf_legacy_window_get_pid              (BamfLegacyWindow *self);
 
 void               bamf_legacy_window_get_geometry         (BamfLegacyWindow *self,
                                                             gint *x, gint *y,
@@ -138,9 +146,17 @@ const char       * bamf_legacy_window_get_class_name       (BamfLegacyWindow *se
 
 const char       * bamf_legacy_window_get_name             (BamfLegacyWindow *self);
 
-const char       * bamf_legacy_window_save_mini_icon       (BamfLegacyWindow *self);
+const char       * bamf_legacy_window_get_role             (BamfLegacyWindow *self);
 
-char             * bamf_legacy_window_get_exec_string      (BamfLegacyWindow *self);
+const char       * bamf_legacy_window_get_exec_string      (BamfLegacyWindow *self);
+
+const char       * bamf_legacy_window_get_working_dir      (BamfLegacyWindow *self);
+
+char             * bamf_legacy_window_save_mini_icon       (BamfLegacyWindow *self);
+
+GFile            * bamf_legacy_window_get_saved_mini_icon  (BamfLegacyWindow *self);
+
+char             * bamf_legacy_window_get_process_name     (BamfLegacyWindow *self);
 
 BamfLegacyWindow * bamf_legacy_window_get_transient        (BamfLegacyWindow *self);
 
@@ -158,3 +174,4 @@ void               bamf_legacy_window_reopen               (BamfLegacyWindow *se
 BamfLegacyWindow * bamf_legacy_window_new                  (WnckWindow *legacy_window);
 
 #endif
+
