@@ -25,30 +25,30 @@
 #include "bamf-application.h"
 #include "bamf-window.h"
 #include "bamf-legacy-window.h"
-#include "bamf-unity-webapps-observer.h"
 
 struct _BamfMatcherPrivate
 {
   GArray          * bad_prefixes;
   GArray          * good_prefixes;
-  GArray          * known_pids;
   GHashTable      * desktop_id_table;
   GHashTable      * desktop_file_table;
   GHashTable      * desktop_class_table;
   GHashTable      * registered_pids;
   GHashTable      * opened_closed_paths_table;
+  GList           * known_pids;
   GList           * views;
   GList           * monitors;
   GList           * favorites;
+  GList           * no_display_desktop;
   BamfView        * active_app;
   BamfView        * active_win;
   guint             dispatch_changes_id;
-  
-  BamfUnityWebappsObserver *webapps_observer;
 };
 
 BamfApplication * bamf_matcher_get_application_by_desktop_file (BamfMatcher *self, const char *desktop_file);
 BamfApplication * bamf_matcher_get_application_by_xid (BamfMatcher *self, guint xid);
-char * bamf_matcher_get_trimmed_exec (BamfMatcher *self, const char *exec);
+char * get_exec_overridden_desktop_file (const char *exec);
+
+gboolean is_autostart_desktop_file (const gchar *desktop_file);
 
 #endif
