@@ -32,6 +32,8 @@ enum
 {
   PROP_0,
 
+  PROP_NAME,
+  PROP_ICON,
   PROP_ACTIVE,
   PROP_RUNNING,
   PROP_URGENT,
@@ -41,6 +43,8 @@ enum
 enum
 {
   CLOSED_INTERNAL,
+  CHILD_ADDED_INTERNAL,
+  CHILD_REMOVED_INTERNAL,
   EXPORTED,
 
   LAST_SIGNAL,
@@ -48,16 +52,47 @@ enum
 
 static guint view_signals[LAST_SIGNAL] = { 0 };
 
+typedef struct _BamfViewPropCache
+{
+  /* FIXME: temporary cache these properties until we don't export the view
+   * to the bus, we need this until the skeleton won't be smart enough to emit
+   * signals as soon as the object is exported */
+  gboolean running;
+  gboolean user_visible;
+  gboolean urgent;
+  gboolean active;
+
+  gchar *name;
+  gchar *icon;
+} BamfViewPropCache;
+
 struct _BamfViewPrivate
 {
-  BamfDBusItemView *dbus_iface;
-  char * name;
+  BamfDBusItemView * dbus_iface;
+  BamfViewPropCache * props;
   char * path;
   GList * children;
   GList * parents;
   gboolean closed;
+
+  /* FIXME: remove this as soon as we move to properties on library as well */
+  guint active_changed_idle;
 };
 
+static gboolean
+on_active_changed_idle (gpointer data)
+{
+  g_return_val_if_fail (BAMF_IS_VIEW (data), FALSE);
+
+  BamfView *self = BAMF_VIEW (data);
+  gboolean active = bamf_view_is_active (self);
+
+  g_signal_emit_by_name (self, "active-changed", active);
+  self->priv->active_changed_idle = 0;
+
+  return FALSE;
+}
+
 static void
 bamf_view_active_changed (BamfView *view, gboolean active)
 {
@@ -70,22 +105,28 @@ bamf_view_active_changed (BamfView *view, gboolean active)
     }
 
   if (emit)
-    g_signal_emit_by_name (view, "active-changed", active);
+    {
+      if (view->priv->active_changed_idle)
+        g_source_remove (view->priv->active_changed_idle);
+
+      guint idle = g_idle_add_full (G_PRIORITY_DEFAULT, on_active_changed_idle, view, NULL);
+      view->priv->active_changed_idle = idle;
+    }
 }
 
 static void
-bamf_view_name_changed (BamfView *view,
-                        const gchar *old_name,
-                        const gchar *new_name)
+bamf_view_name_changed (BamfView *view, const gchar *new_name)
 {
   g_return_if_fail (BAMF_IS_VIEW (view));
 
+  const gchar *old_name = bamf_view_get_name (view);
   g_signal_emit_by_name (view, "name-changed", old_name, new_name);
+}
 
-  if (view->priv->name)
-    g_free (view->priv->name);
-
-  view->priv->name = g_strdup (new_name);
+static void
+bamf_view_icon_changed (BamfView *view, const gchar *new_icon)
+{
+  g_object_notify (G_OBJECT (view), "icon");
 }
 
 static void
@@ -278,15 +319,16 @@ bamf_view_add_child (BamfView *view,
   if (BAMF_VIEW_GET_CLASS (view)->child_added)
     BAMF_VIEW_GET_CLASS (view)->child_added (view, child);
 
+  g_signal_emit (view, view_signals[CHILD_ADDED_INTERNAL], 0, child);
+
   added = bamf_view_get_path (child);
   g_signal_emit_by_name (view, "child-added", added);
 }
 
 void
-bamf_view_remove_child (BamfView *view,
-                        BamfView *child)
+bamf_view_remove_child (BamfView *view, BamfView *child)
 {
-  const char * removed;
+  const char *removed;
 
   g_return_if_fail (BAMF_IS_VIEW (view));
   g_return_if_fail (BAMF_IS_VIEW (child));
@@ -297,129 +339,134 @@ bamf_view_remove_child (BamfView *view,
   view->priv->children = g_list_remove (view->priv->children, child);
   child->priv->parents = g_list_remove (child->priv->parents, view);
 
+  g_signal_emit (view, view_signals[CHILD_REMOVED_INTERNAL], 0, child);
+
   removed = bamf_view_get_path (child);
   g_signal_emit_by_name (view, "child-removed", removed);
 
-  // Do this by hand so we can pass and object instead of a string
+  /* Do this by hand so we can pass and object instead of a string */
   if (BAMF_VIEW_GET_CLASS (view)->child_removed)
     BAMF_VIEW_GET_CLASS (view)->child_removed (view, child);
 }
 
+#define BAMF_VIEW_GET_PROPERTY(v, property, ret_val)                          \
+  g_return_val_if_fail (BAMF_IS_VIEW (v), ret_val);                           \
+                                                                              \
+  if (v->priv->props)                                                         \
+    return v->priv->props->property;                                          \
+                                                                              \
+  return _bamf_dbus_item_view_get_##property (v->priv->dbus_iface);
+
+#define BAMF_VIEW_SET_BOOL_PROPERTY(v, property)                              \
+  g_return_if_fail (BAMF_IS_VIEW (v));                                        \
+                                                                              \
+  if (property == bamf_view_is_##property (v))                                \
+    return;                                                                   \
+                                                                              \
+  if (v->priv->props)                                                         \
+    {                                                                         \
+      v->priv->props->property = property;                                    \
+    }                                                                         \
+  else                                                                        \
+    {                                                                         \
+      _bamf_dbus_item_view_set_##property (v->priv->dbus_iface, property);    \
+    }                                                                         \
+                                                                              \
+  bamf_view_##property##_changed (v, property);
+
+#define BAMF_VIEW_SET_STRING_PROPERTY(v, property)                            \
+  g_return_if_fail (BAMF_IS_VIEW (v));                                        \
+                                                                              \
+  const gchar *current_value = bamf_view_get_##property (v);                  \
+                                                                              \
+  if (current_value == property || g_strcmp0 (current_value, property) == 0)  \
+    return;                                                                   \
+                                                                              \
+  bamf_view_##property##_changed (v, property);                               \
+                                                                              \
+  if (v->priv->props)                                                         \
+    {                                                                         \
+      g_free (v->priv->props->property);                                      \
+      v->priv->props->property = g_strdup (property);                         \
+    }                                                                         \
+  else                                                                        \
+    {                                                                         \
+      _bamf_dbus_item_view_set_##property (v->priv->dbus_iface, property);    \
+    }
+
 gboolean
 bamf_view_is_active (BamfView *view)
 {
-  g_return_val_if_fail (BAMF_IS_VIEW (view), FALSE);
-
-  return bamf_dbus_item_view_get_active (view->priv->dbus_iface);
+  BAMF_VIEW_GET_PROPERTY (view, active, FALSE);
 }
 
 void
 bamf_view_set_active (BamfView *view,
                       gboolean active)
 {
-  g_return_if_fail (BAMF_IS_VIEW (view));
-
-  if (active == bamf_view_is_active (view))
-    return;
-
-  bamf_dbus_item_view_set_active (view->priv->dbus_iface, active);
-  bamf_view_active_changed (view, active);
+  BAMF_VIEW_SET_BOOL_PROPERTY (view, active);
 }
 
 gboolean
 bamf_view_is_urgent (BamfView *view)
 {
-  g_return_val_if_fail (BAMF_IS_VIEW (view), FALSE);
-
-  return bamf_dbus_item_view_get_urgent (view->priv->dbus_iface);
+  BAMF_VIEW_GET_PROPERTY (view, urgent, FALSE);
 }
 
 void
 bamf_view_set_urgent (BamfView *view,
                        gboolean urgent)
 {
-  g_return_if_fail (BAMF_IS_VIEW (view));
-
-  if (urgent == bamf_view_is_urgent (view))
-    return;
-
-  bamf_dbus_item_view_set_urgent (view->priv->dbus_iface, urgent);
-  bamf_view_urgent_changed (view, urgent);
+  BAMF_VIEW_SET_BOOL_PROPERTY (view, urgent);
 }
 
 gboolean
 bamf_view_is_running (BamfView *view)
 {
-  g_return_val_if_fail (BAMF_IS_VIEW (view), FALSE);
-
-  return bamf_dbus_item_view_get_running (view->priv->dbus_iface);
+ BAMF_VIEW_GET_PROPERTY (view, running, FALSE);
 }
 
 void
 bamf_view_set_running (BamfView *view,
                        gboolean running)
 {
-  g_return_if_fail (BAMF_IS_VIEW (view));
-
-  if (running == bamf_view_is_running (view))
-    return;
-
-  bamf_dbus_item_view_set_running (view->priv->dbus_iface, running);
-  bamf_view_running_changed (view, running);
+  BAMF_VIEW_SET_BOOL_PROPERTY (view, running);
 }
 
 gboolean
-bamf_view_user_visible (BamfView *view)
+bamf_view_is_user_visible (BamfView *view)
 {
-  g_return_val_if_fail (BAMF_IS_VIEW (view), FALSE);
-
-  return bamf_dbus_item_view_get_user_visible (view->priv->dbus_iface);
+  BAMF_VIEW_GET_PROPERTY (view, user_visible, FALSE);
 }
 
 void
 bamf_view_set_user_visible (BamfView *view, gboolean user_visible)
 {
-  g_return_if_fail (BAMF_IS_VIEW (view));
-
-  if (user_visible == bamf_view_user_visible (view))
-    return;
-
-  bamf_dbus_item_view_set_user_visible (view->priv->dbus_iface, user_visible);
-  bamf_view_user_visible_changed (view, user_visible);
+  BAMF_VIEW_SET_BOOL_PROPERTY (view, user_visible);
 }
 
 const char *
 bamf_view_get_icon (BamfView *view)
 {
-  g_return_val_if_fail (BAMF_IS_VIEW (view), NULL);
+  BAMF_VIEW_GET_PROPERTY (view, icon, NULL);
+}
 
-  if (BAMF_VIEW_GET_CLASS (view)->get_icon)
-    return BAMF_VIEW_GET_CLASS (view)->get_icon (view);
-
-  return NULL;
+void
+bamf_view_set_icon (BamfView *view, const char *icon)
+{
+  BAMF_VIEW_SET_STRING_PROPERTY (view, icon);
 }
 
 const char *
 bamf_view_get_name (BamfView *view)
 {
-  g_return_val_if_fail (BAMF_IS_VIEW (view), NULL);
-
-  if (BAMF_VIEW_GET_CLASS (view)->get_name)
-    return BAMF_VIEW_GET_CLASS (view)->get_name (view);
-
-  return view->priv->name;
+  BAMF_VIEW_GET_PROPERTY (view, name, NULL);
 }
 
 void
-bamf_view_set_name (BamfView *view,
-                    const char * name)
+bamf_view_set_name (BamfView *view, const char *name)
 {
-  g_return_if_fail (BAMF_IS_VIEW (view));
-
-  if (!g_strcmp0 (name, view->priv->name))
-    return;
-
-  bamf_view_name_changed (view, view->priv->name, name);
+  BAMF_VIEW_SET_STRING_PROPERTY (view, name);
 }
 
 const char *
@@ -433,15 +480,47 @@ bamf_view_get_view_type (BamfView *view)
   return "view";
 }
 
-char *
+static char *
 bamf_view_get_stable_bus_name (BamfView *view)
 {
   g_return_val_if_fail (BAMF_IS_VIEW (view), NULL);
 
   if (BAMF_VIEW_GET_CLASS (view)->stable_bus_name)
     return BAMF_VIEW_GET_CLASS (view)->stable_bus_name (view);
-    
-  return g_strdup_printf ("view%p", view);
+
+  return g_strdup_printf ("view/%p", view);
+}
+
+static void
+bamf_view_cached_properties_clear (BamfView *view)
+{
+  if (!view->priv->props)
+    return;
+
+  g_free (view->priv->props->name);
+  g_free (view->priv->props->icon);
+  g_free (view->priv->props);
+  view->priv->props = NULL;
+}
+
+static void
+bamf_view_cached_properties_notify (BamfView *view)
+{
+  if (!view->priv->props || !bamf_view_is_on_bus (view))
+    return;
+
+  /* Temporary disable the cache so that cached values will be set on the skeleton */
+  BamfViewPropCache *cache = view->priv->props;
+  view->priv->props = NULL;
+
+  bamf_view_set_name (view, cache->name);
+  bamf_view_set_icon (view, cache->icon);
+  bamf_view_set_active (view, cache->active);
+  bamf_view_set_running (view, cache->running);
+  bamf_view_set_user_visible (view, cache->user_visible);
+  bamf_view_set_urgent (view, cache->urgent);
+
+  view->priv->props = cache;
 }
 
 const char *
@@ -456,8 +535,9 @@ bamf_view_export_on_bus (BamfView *view, GDBusConnection *connection)
 
   if (!view->priv->path)
     {
+      gboolean exported = TRUE;
       char *stable_name = bamf_view_get_stable_bus_name (view);
-      path = g_strdup_printf ("%s/%s", BAMF_DBUS_PATH, stable_name);
+      path = g_strconcat (BAMF_DBUS_BASE_PATH, "/", stable_name, NULL);
       g_free (stable_name);
 
       BAMF_VIEW_GET_CLASS (view)->names = g_list_prepend (BAMF_VIEW_GET_CLASS (view)->names, path);
@@ -467,19 +547,29 @@ bamf_view_export_on_bus (BamfView *view, GDBusConnection *connection)
 
       /* The dbus object interface list is in reversed order, we try to export
        * the interfaces in bottom to top order (BamfView should be the first) */
-      for (l = g_list_last(ifaces); l; l = l->prev)
+      for (l = g_list_last (ifaces); l; l = l->prev)
         {
           g_dbus_interface_skeleton_export (G_DBUS_INTERFACE_SKELETON (l->data),
                                             connection, path, &error);
+          if (error)
+            {
+              g_critical ("Can't register BAMF view interface: %s", error->message);
+              g_clear_error (&error);
+              exported = FALSE;
+            }
         }
 
-      if (error)
-        {
-          g_critical ("Can't register BAMF view: %s", error->message);
-          g_error_free (error);
-        }
-      else
+      if (exported)
         {
+          /* FIXME: if we change the properties before that the view has been
+           * exported, the skeleton doesn't emit the proper signals to notify
+           * the proxy that the values have been changed, and this causes
+           * the properties not to be updated on the client side.
+           * So we store the values locally until the proxy is not exported,
+           * then we notify our clients. */
+          bamf_view_cached_properties_notify (view);
+          bamf_view_cached_properties_clear (view);
+
           g_signal_emit (view, view_signals[EXPORTED], 0);
         }
 
@@ -496,10 +586,13 @@ bamf_view_is_on_bus (BamfView *view)
   GDBusInterfaceSkeleton *dbus_iface;
   const gchar *exported_path;
 
+  if (!view->priv->path)
+    return FALSE;
+
   dbus_iface = G_DBUS_INTERFACE_SKELETON (view->priv->dbus_iface);
   exported_path = g_dbus_interface_skeleton_get_object_path (dbus_iface);
 
-  return (view->priv->path != NULL && exported_path != NULL);
+  return (exported_path != NULL);
 }
 
 static void
@@ -558,6 +651,7 @@ static void
 on_view_closed (BamfView *view, gpointer _not_used)
 {
   g_return_if_fail (BAMF_IS_VIEW (view));
+  g_dbus_object_skeleton_flush (G_DBUS_OBJECT_SKELETON (view));
   g_signal_emit_by_name (view->priv->dbus_iface, "closed");
 }
 
@@ -578,7 +672,7 @@ on_dbus_handle_user_visible (BamfDBusItemView *interface,
                              GDBusMethodInvocation *invocation,
                              BamfView *view)
 {
-  gboolean user_visible = bamf_view_user_visible (view);
+  gboolean user_visible = bamf_view_is_user_visible (view);
   g_dbus_method_invocation_return_value (invocation,
                                          g_variant_new ("(b)", user_visible));
 
@@ -672,25 +766,6 @@ bamf_view_dispose (GObject *object)
 {
   BamfView *view = BAMF_VIEW (object);
   BamfViewPrivate *priv = view->priv;
-  GList *ifaces, *l;
-
-  ifaces = g_dbus_object_get_interfaces (G_DBUS_OBJECT (view));
-
-  for (l = ifaces; l; l = l->next)
-    {
-      GDBusInterfaceSkeleton *iface = G_DBUS_INTERFACE_SKELETON (l->data);
-
-      if (g_dbus_interface_skeleton_get_object_path (iface))
-        g_dbus_interface_skeleton_unexport (iface);
-    }
-
-  g_list_free_full (ifaces, g_object_unref);
-
-  if (priv->name)
-    {
-      g_free (priv->name);
-      priv->name = NULL;
-    }
 
   if (priv->path)
     {
@@ -710,6 +785,15 @@ bamf_view_dispose (GObject *object)
       priv->parents = NULL;
     }
 
+  if (priv->active_changed_idle)
+    {
+      g_source_remove (priv->active_changed_idle);
+      priv->active_changed_idle = 0;
+    }
+
+  bamf_view_cached_properties_clear (view);
+  g_dbus_object_skeleton_flush (G_DBUS_OBJECT_SKELETON (view));
+
   G_OBJECT_CLASS (bamf_view_parent_class)->dispose (object);
 }
 
@@ -730,6 +814,12 @@ bamf_view_get_property (GObject *object, guint property_id, GValue *value, GPara
 
   switch (property_id)
     {
+      case PROP_NAME:
+        g_value_set_string (value, bamf_view_get_name (view));
+        break;
+      case PROP_ICON:
+        g_value_set_string (value, bamf_view_get_icon (view));
+        break;
       case PROP_ACTIVE:
         g_value_set_boolean (value, bamf_view_is_active (view));
         break;
@@ -737,7 +827,7 @@ bamf_view_get_property (GObject *object, guint property_id, GValue *value, GPara
         g_value_set_boolean (value, bamf_view_is_urgent (view));
         break;
       case PROP_USER_VISIBLE:
-        g_value_set_boolean (value, bamf_view_user_visible (view));
+        g_value_set_boolean (value, bamf_view_is_user_visible (view));
         break;
       case PROP_RUNNING:
         g_value_set_boolean (value, bamf_view_is_running (view));
@@ -763,7 +853,8 @@ bamf_view_init (BamfView * self)
   self->priv = BAMF_VIEW_GET_PRIVATE (self);
 
   /* Initializing the dbus interface */
-  self->priv->dbus_iface = bamf_dbus_item_view_skeleton_new ();
+  self->priv->dbus_iface = _bamf_dbus_item_view_skeleton_new ();
+  self->priv->props = g_new0 (BamfViewPropCache, 1);
 
   /* We need to connect to the object own signals to redirect them to the dbus
    * interface                                                                */
@@ -805,8 +896,8 @@ bamf_view_init (BamfView * self)
                     G_CALLBACK (on_dbus_handle_children), self);
 
   /* Setting the interface for the dbus object */
-  bamf_dbus_item_object_skeleton_set_view (BAMF_DBUS_ITEM_OBJECT_SKELETON (self),
-                                           self->priv->dbus_iface);
+  _bamf_dbus_item_object_skeleton_set_view (BAMF_DBUS_ITEM_OBJECT_SKELETON (self),
+                                            self->priv->dbus_iface);
 }
 
 static void
@@ -828,6 +919,8 @@ bamf_view_class_init (BamfViewClass * klass)
 
   /* Overriding the properties defined in the interface, this is needed
    * but we actually don't use these properties, as we act like a proxy       */
+  g_object_class_override_property (object_class, PROP_NAME, "name");
+  g_object_class_override_property (object_class, PROP_ICON, "icon");
   g_object_class_override_property (object_class, PROP_ACTIVE, "active");
   g_object_class_override_property (object_class, PROP_URGENT, "urgent");
   g_object_class_override_property (object_class, PROP_RUNNING, "running");
@@ -836,16 +929,32 @@ bamf_view_class_init (BamfViewClass * klass)
   view_signals [CLOSED_INTERNAL] =
     g_signal_new ("closed-internal",
                   G_OBJECT_CLASS_TYPE (klass),
-                  0,
-                  0, NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
+                  G_SIGNAL_RUN_LAST,
+                  G_STRUCT_OFFSET (BamfViewClass, closed_internal),
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
 
+  view_signals [CHILD_ADDED_INTERNAL] =
+    g_signal_new ("child-added-internal",
+                  G_OBJECT_CLASS_TYPE (klass),
+                  G_SIGNAL_RUN_FIRST,
+                  G_STRUCT_OFFSET (BamfViewClass, child_added_internal),
+                  NULL, NULL, NULL,
+                  G_TYPE_NONE, 1, BAMF_TYPE_VIEW);
+
+  view_signals [CHILD_REMOVED_INTERNAL] =
+    g_signal_new ("child-removed-internal",
+                  G_OBJECT_CLASS_TYPE (klass),
+                  G_SIGNAL_RUN_FIRST,
+                  G_STRUCT_OFFSET (BamfViewClass, child_removed_internal),
+                  NULL, NULL, NULL,
+                  G_TYPE_NONE, 1, BAMF_TYPE_VIEW);
+
   view_signals [EXPORTED] =
     g_signal_new ("exported",
                   G_OBJECT_CLASS_TYPE (klass),
-                  0,
-                  0, NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
+                  G_SIGNAL_RUN_FIRST,
+                  G_STRUCT_OFFSET (BamfViewClass, exported),
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
 }
