@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010 Canonical Ltd
+ * Copyright (C) 2010-2011 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -14,6 +14,7 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * Authored by: Jason Smith <jason.smith@canonical.com>
+ *              Marco Trevisan (Treviño) <3v1n0@ubuntu.com>
  *
  */
 
@@ -21,16 +22,10 @@
 #define __MAIN_H__
 
 #include <stdio.h>
-#include <gtk/gtk.h>
-#include <gdk/gdk.h>
-#include <dbus/dbus.h>
-#include <dbus/dbus-glib.h>
-#include <dbus/dbus-glib-lowlevel.h>
 #include <glib.h>
+#include <gio/gio.h>
+#include <glibtop.h>
 #include <sys/types.h>
 #include <unistd.h>
-#include <glibtop.h>
-
-
 
 #endif
